/* $Novell: ldap_ssl.h,v 1.32 2003/05/13 13:52:32 $ 
 ******************************************************************************
 * 
 *	Copyright (c) 2004 Novell, Inc.
 *	All rights reserved.
 *
 *	Redistribution and use in source and binary forms, with or without 
 *	modification, are permitted provided that the following conditions are met:
 *
 *	Redistributions of source code must retain the above copyright notice, this 
 *	list of conditions and the following disclaimer. 
 *
 *	Redistributions in binary form must reproduce the above copyright notice, 
 *	this list of conditions and the following disclaimer in the documentation 
 *	and or other materials provided with the distribution.
 *
 *	Neither the name of Novell nor the names of its contributors may be used to 
 *	endorse or promote products derived from this software without specific prior
 *	written permission. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 *	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 *	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************
 */

#if !defined(LDAP_SSL_H)
#define LDAP_SSL_H

#include <ldap.h>
#include <sys/select.h>
#include <sys/socket.h>
#ifdef __cplusplus
extern "C" {
#endif

#define LDAPSSL_SUCCESS          0
#define LDAPSSL_ERROR           -1

#define LDAPSSL_VERIFY_NONE   0x00
#define LDAPSSL_VERIFY_SERVER 0x01

#define LDAPSSL_FILETYPE_B64     1
#define LDAPSSL_FILETYPE_DER     2
#define LDAPSSL_BUFFTYPE_B64     3
#define LDAPSSL_BUFFTYPE_DER     4
#define LDAPSSL_FILETYPE_P12     5
#define LDAPSSL_BUFFTYPE_P12     6

#define LDAPSSL_CERT_FILETYPE_B64     LDAPSSL_FILETYPE_B64
#define LDAPSSL_CERT_FILETYPE_DER     LDAPSSL_FILETYPE_DER
#define LDAPSSL_CERT_BUFFTYPE_B64     LDAPSSL_BUFFTYPE_B64
#define LDAPSSL_CERT_BUFFTYPE_DER     LDAPSSL_BUFFTYPE_DER

#define LDAPSSL_CERT_ATTR_ISSUER            1
#define LDAPSSL_CERT_ATTR_SUBJECT           2
#define LDAPSSL_CERT_ATTR_VALIDITY_PERIOD   3

#define LDAPSSL_CERT_GET_STATUS           100

/*
 * Certificate Status
 */
/* unable to get issuer certificate */
#define UNABLE_TO_GET_ISSUER_CERT           2
/* unable to decode issuer public key */
#define UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY  6
/* certificate signature failure */
#define CERT_SIGNATURE_FAILURE              7
/* certificate is not yet valid */
#define CERT_NOT_YET_VALID                  9	
/* CRL is not yet valid */
#define CERT_HAS_EXPIRED                   10
/* format error in certificate's notBefore field */
#define ERROR_IN_CERT_NOT_BEFORE_FIELD     13
/* format error in certificate's notAfter field */
#define ERROR_IN_CERT_NOT_AFTER_FIELD      14
/* self signed certificate */
#define DEPTH_ZERO_SELF_SIGNED_CERT        18
/* self signed certificate in certificate chain */
#define SELF_SIGNED_CERT_IN_CHAIN          19
/* unable to get local issuer certificate */
#define UNABLE_TO_GET_ISSUER_CERT_LOCALLY  20
/* unable to verify the first certificate */
#define UNABLE_TO_VERIFY_LEAF_SIGNATURE    21
/* invalid CA certificate */
#define INVALID_CA                         24
/* path length constraint exceeded */
#define PATH_LENGTH_EXCEEDED               25
/* unsupported certificate purpose */
#define INVALID_PURPOSE                    26
/* certificate not trusted */
#define CERT_UNTRUSTED                     27
/* certificate rejected */
#define CERT_REJECTED                      28


#define LDAPSSL_CERT_UTC_TIME 1
#define LDAPSSL_CERT_GEN_TIME 2

#define LDAPSSL_CERT_ACCEPT 0
#define LDAPSSL_CERT_REJECT -1

#define LDAP_OPT_IO_FUNCS	0x7001
#if defined(NPLAT_NLM)
	#define LIBCALL
#elif defined(_WIN64)
	#define LIBCALL __stdcall
#elif defined(_WIN32)
	#define LIBCALL __stdcall
#else
/* LIBCALL should be NULL if not specified. Use __cdecl */
	#define LIBCALL
#endif

typedef struct ldap_io_funcs {
	ber_socket_t (LIBCALL *io_socket) (int,int,int,LDAP *);
	int (LIBCALL *io_prepare)  (ber_socket_t, LDAP *);
	int (LIBCALL *io_connect)  (ber_socket_t, struct sockaddr *,int, LDAP *);
	int (LIBCALL *io_read)	   (ber_socket_t, void *, ber_len_t, LDAP *);
	int (LIBCALL *io_write)    (ber_socket_t, void *, ber_len_t, LDAP *);
#ifdef HAVE_POLL
	int (LIBCALL *io_select)   (int, struct pollfd *, fd_set *, struct timeval *, LDAP *);
#else
	int (LIBCALL *io_select)   (int, fd_set *, fd_set *, fd_set *, struct timeval *, LDAP *);
#endif 
	int (LIBCALL *io_ioctl)    (ber_socket_t, long, void *, LDAP *);
	int (LIBCALL *io_close)    (ber_socket_t, LDAP *);
}LDAPIOFuncs;

typedef struct _LDAPSSL_Private_Key
{
    unsigned long    length;
    void            *data;

} LDAPSSL_Private_Key, *pLDAPSSL_Private_Key;

typedef struct _LDAPSSL_Cert
{
    unsigned long    length;
    void            *data;

} LDAPSSL_Cert, *pLDAPSSL_Cert;

typedef struct _LDAPSSL_Cert_Validity_Period
{
   char  notBeforeTime[40];
   int   notBeforeType;
   char  notAfterTime[40];
   int   notAfterType;

} LDAPSSL_Cert_Validity_Period, *pLDAPSSL_Cert_Validity_Period;


/* APIs */
LDAP_F(int) ldapssl_client_init
(
   const char *certFile,
   void *reserved
);

LDAP_F(LDAP *) ldapssl_init
(
   const char *defhost,
   int defport,
   int defsecure
);

LDAP_F(int) ldapssl_install_routines
(
   LDAP *ld
);

LDAP_F(int) ldapssl_client_deinit
(
   void
);

LDAP_F(int) ldapssl_add_trusted_cert
(
   void *cert,
   int   type
);

LDAP_F(int) ldapssl_set_verify_mode
(
   int   mode
);

LDAP_F(int) ldapssl_get_verify_mode
(
   int   *mode
);

LDAP_F(int) ldapssl_set_verify_callback
(
   int (LIBCALL *certVerifyFunc)(void *)
);

LDAP_F(int) ldapssl_get_cert_attribute
(
   void          *certHandle,
   int           attrID,
   void          *value,
   int           *length
);

LDAP_F(int) ldapssl_get_cert
(
   void          *certHandle,
   int            type,
   LDAPSSL_Cert  *cert
);

LDAP_F(int) ldapssl_set_client_private_key
(
   void *key,
   int   type,
   void *password
);

LDAP_F(int) ldapssl_set_client_cert
(
   void *cert,
   int   type,
   void *password
);

LDAP_F(int) ldapssl_start_tls
(
   LDAP *ld
);

LDAP_F(int) ldapssl_stop_tls
(
   LDAP *ld
);

#ifdef __cplusplus
}
#endif

#endif /* !defined(LDAP_SSL_H) */
